@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

.@{ant-prefix}-form {
  // 因为 light-filter 不需要定宽
  > div:not(.@{ant-prefix}-pro-form-light-filter) {
    .pro-field {
      max-width: 100%;
      // 适用于短数字，短文本或者选项
      &-xs {
        width: 104px;
      }
      &-s {
        width: 216px;
      }
      // 适用于较短字段录入、如姓名、电话、ID 等。
      &-sm {
        width: 216px;
      }
      &-m {
        width: 328px;
      }
      // 标准宽度，适用于大部分字段长度
      &-md {
        width: 328px;
      }
      &-l {
        width: 440px;
      }
      // 适用于较长字段录入，如长网址、标签组、文件路径等。
      &-lg {
        width: 440px;
      }
      // 适用于长文本录入，如长链接、描述、备注等，通常搭配自适应多行输入框或定高文本域使用。
      &-xl {
        width: 552px;
      }
    }
  }
}
