@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-field-select-light-select: ~'@{ant-prefix}-pro-field-select-light-select';

.@{pro-field-select-light-select} {
  .@{ant-prefix}-select {
    position: absolute;
    width: 153px;
    height: 28px;
    visibility: hidden;
  }
  .@{ant-prefix}-select-selector {
    height: 28px;
  }
  &.@{pro-field-select-light-select}-searchable {
    .@{ant-prefix}-select {
      width: 200px;
    }
    .@{ant-prefix}-select-selector {
      height: 28px;
    }
  }
}
