@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-core-label-tip: ~'@{ant-prefix}-pro-core-label-tip';

.@{pro-core-label-tip} {
  display: inline-flex;
  align-items: center;
  max-width: 100%;

  &-icon {
    display: block;
    margin-left: 4px;
    cursor: pointer;
    &:hover {
      color: @primary-color-hover;
    }
  }

  &-title {
    display: inline-flex;
    flex: 1;
  }

  &-subtitle {
    margin-left: 8px;
    color: @text-color-secondary;
    font-weight: normal;
    font-size: @font-size-base;
    white-space: nowrap;
  }

  &-title-ellipsis {
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
    word-break: keep-all;
  }
}
