@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-field-light-wrapper: ~'@{ant-prefix}-pro-field-light-wrapper';

.@{pro-field-light-wrapper}-collapse-label {
  padding: 1;
}

.@{pro-field-light-wrapper}-container {
  .@{ant-prefix}-form-item {
    margin-bottom: 0;
  }
}
