@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-table-prefix-cls: ~'@{ant-prefix}-pro-table';

.pro-table-tooltip-text span {
  color: @component-background;
}

.@{pro-table-prefix-cls} {
  z-index: 1;

  &:not(:root):fullscreen {
    min-height: 100vh;
    overflow: auto;
    background: @component-background;
  }

  &-extra {
    margin-bottom: 16px;
  }

  &-polling {
    .@{pro-table-prefix-cls}-list-toolbar-setting-item {
      .anticon.anticon-reload {
        transform: rotate(0deg);
        animation: turn 1s linear infinite;
      }
    }
  }

  td.@{ant-prefix}-table-cell {
    > a {
      font-size: @font-size-base;
    }
  }

  .@{ant-prefix}-table
    .@{ant-prefix}-table-tbody
    .@{ant-prefix}-table-wrapper:only-child
    .@{ant-prefix}-table {
    margin: 0;
  }

  .@{ant-prefix}-table.@{ant-prefix}-table-middle .@{pro-table-prefix-cls} {
    margin: -12px -8px;
  }
}

@keyframes turn {
  0% {
    transform: rotate(0deg);
  }
  25% {
    transform: rotate(90deg);
  }
  50% {
    transform: rotate(180deg);
  }
  75% {
    transform: rotate(270deg);
  }
  100% {
    transform: rotate(360deg);
  }
}
