@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-dropdown-prefix-cls: ~'@{ant-prefix}-pro-table-drag';

.@{pro-dropdown-prefix-cls} {
  &-visible-cell {
    display: flex;
    align-items: center;
  }

  &-icon {
    margin-right: 8px;
    color: #999;
    cursor: grab;
  }
}
